/*

  Name		: SMTPMove v1.0
  Meaning      	: Takes the mail from the sendmail-inbox and moves
		  it into the Pop3d-outboxes or to the Gateway.
  Syntax	: SMTPMove [-i InBox] [-m PathToMailaddr] [-u]
  Arguments	: see Helppage in code! near EOF ;-)
  Care about	: the file SENDMAIL.CF must include:
                  Mlocal, P=c:\tcpip\umail\umailer.exe, F=lsDFP, S=10, R=20, A=-dest c:\tcpip\etc\mail -to $u
                  and of course UMAILER must exist! 
                  (Warp Bonuspack -> UMail lite)
  ToDo          : I have to write an own MAILER (SMAILER!?) for use with
                  SMTPMove.
  Author	: Acki@u9000mst.nez.telekom.de

*/

#define INCL_DOSPROCESS
#include <io.h>
#include <dos.h>
#include <env.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>

void  intropg( void );
void   helppg( void );
void   strcln( char *pc );
char  fcopyau( char *pcOrig, char *pcDest, char *pcUser );
char    fmove( char *pcFile, char *pcPath );
char  fmoveau( char *pcFile, char *pcPath, char *pcUser );
char   fcopy_( char *pcFile, char *pcPath );
char fcopyau_( char *pcFile, char *pcPath, char *pcUser );
char  *job_id( int update );

int main( int argc, char **argv )
{
    char           ccount, cUUCP, cDebug, uniq;
    char           *pcTmp, *job;
    char           *pcPOP3e, *pcSMTPe, *pcHOSTe, *pcUSPLe;
    char           pcPOP3[256],  pcSMTP[256],   pcDefBox[256];
    char           pcMail[256],  pcUser[32],   pcSearch[256];
    char           pcHOST[32],  pcDstBox[256], pcDefUsr[256];
    char           pcUSPL[256],  pcZeile[256],  pcFound[256];
    char           pcRdName[32], pcRdFile[256], pcRxName[32];
    char           pcRxFile[256], pcFromUser[32], pcFromHost[32];
    unsigned       rc;
    struct find_t  fileinfo;
    FILE           *pFPOP3, *pFFound, *pFRxFile, *pFMail, *pF;
    
    // test for environment
    pcPOP3e=getenv( "MAILADDR" );
    pcSMTPe=getenv( "SMTPMAIL" );
    pcHOSTe=getenv( "HOSTNAME" );
    pcUSPLe=getenv( "UUCPSPOOL" );
    
    // get environment if set
    if( pcPOP3e )
        strcpy( pcPOP3, pcPOP3e );
    if( pcSMTPe )
        strcpy( pcSMTP, pcSMTPe );
    if( pcHOSTe )
        strcpy( pcHOST, pcHOSTe );
    if( pcUSPLe )
        strcpy( pcUSPL, pcUSPLe );
    
    // defaults
    cUUCP=0;
    cDebug=0;

    // override environment if argument found
    for( ccount=1; ccount<argc; ccount++ )
    {
        // -dbg signals gebug-mode (more information)
        if( !strcmp( argv[ccount], "-dbg" ) )
            cDebug=1;
        
        // -m (overrides envi MAILADDR) path to mailaddr-file
        if( !strcmp( argv[ccount], "-m" ) )
            strcpy( pcPOP3, argv[ccount+1] );
        
        // -i (overrides envi SMTPMAIL) path to incoming SMTP-Mail
        if( !strcmp( argv[ccount], "-i" ) )
            strcpy( pcSMTP, argv[ccount+1] );
        
        // -u signals forwarding mail by UUCP for unknown user
        if( !strcmp( argv[ccount], "-u" ) )
            cUUCP=1;
        
        // -h (overrides envi HOSTNAME) the name of this Host
        if( !strcmp( argv[ccount], "-h" ) )
            strcpy( pcHOST, argv[ccount+1] );
        
        // -s the path to the UUCP spooldir
        if( !strcmp( argv[ccount], "-s" ) )
            strcpy( pcUSPL, argv[ccount+1] );
    };
    
    // helppage if arguments and/or env not set correctly
    if( (!cUUCP&&(!pcPOP3[0]||!pcSMTP[0])) || (cUUCP&&(!pcPOP3[0]||!pcSMTP[0]||!pcHOST[0]||!pcUSPL[0])) )
    {
        helppg();
        return(1);
    };
    
    // liddle Intro
    intropg();
    
    // cleanup strings
    strcln( pcPOP3 );
    strcln( pcSMTP );
    strcln( pcUSPL );
    strlwr( pcHOST );
    
    // show arguments if -dbg set
    if( cDebug )
    {
        printf("-m : %s\n", pcPOP3);
        printf("-i : %s\n", pcSMTP);
        if(!cUUCP)
            printf("-u : false\n");
        else
        {
            printf("-u : true\n");
            printf("-h : %s\n", pcHOST);
            printf("-s : %s\n", pcUSPL);
        };
        printf("\n");
    };

    // generate full qualified filename
    strcat( pcPOP3, "/mailaddr" );
    
    // open mailaddr-file
    pFPOP3=fopen( pcPOP3, "rb" );
    if( pFPOP3 )
    {
        // read first line
        fgets( pcZeile , 255, pFPOP3 );
        // get second word in this line as default mailbox for unknown user.
        pcTmp=strtok( pcZeile, " \t" );
        strcpy( pcDefUsr, pcTmp );
        pcTmp=strtok(    NULL, " \t\n\r" );
        strcpy( pcDefBox, pcTmp );
        // close mailaddr-file
        fclose( pFPOP3 );
    }
    else
    {
        // errormsg if unsuccessfull
        printf( "   Unable to open %s ! ... exit\n\n", pcPOP3 );
        // exit with errorlevel 2
        return(2);
    };
    
    // ...search for mail in SMTPinBox...
    // generate full qualified filename
    strcpy( pcSearch, pcSMTP );
    strcat( pcSearch, "/*.uml" );
    
    // get first hit
    rc = _dos_findfirst( pcSearch, _A_NORMAL, &fileinfo );
    
    // repeat until no more
    while( rc == 0 )
    {
        // generate full qualified filename
        strcpy( pcFound, pcSMTP );
        strcat( pcFound, "/" );
        strcat( pcFound, fileinfo.name );
        
        // open mailctrl-file
        pFFound=fopen( pcFound, "rb" );
        if( pFFound )
        {
            // read first line
            fgets( pcZeile , 255, pFFound );
            
            // close mailctrl-file
            fclose( pFFound );
            
            // get first  word in this as mail-file.
            pcTmp=strtok( pcZeile, " \t" );
            strcpy( pcMail, pcTmp );
            
            // get second word in this as rcpt-name.
            pcTmp=strtok(    NULL, " \t\n\r" );
            strcpy( pcUser, pcTmp );
            
            // is rcpt-name in UUCP-format ?
            if( strrchr( pcUser, '!' ) )
                strcpy( pcUser, strrchr( pcUser, '!' )+1 );
            
            // search for rcpt in mailaddr-file
            pFPOP3=fopen( pcPOP3, "rb" );
            if( pFPOP3 )
            {
                // reset destbox
                pcDstBox[0]=0;
                
                // read while lines
                while( fgets( pcZeile , 255, pFPOP3 ) )
                {
                    // get first word in this line.
                    pcTmp=strtok( pcZeile, " \t" );
                    
                    // User found ?
                    if( !stricmp( pcUser, pcTmp ) )
                    {
                        // get socond word as dest-path
                        pcTmp=strtok(    NULL, " \t\n\r" );
                        strcpy( pcDstBox, pcTmp );
                        break;
                    };
                };
                // close mailaddr-file
                fclose( pFPOP3 );
                
                // move Mail to it's destination
                
                // user known!
                if( pcDstBox[0] )
                {
                    printf( "Moving mail for %s into %s's Pop3d-Box...", pcUser, pcUser );
                    if( !fmove( pcMail, pcDstBox ) )
                    {
                        if( !unlink( pcFound ) )
                            printf( "success\n" );
                    }else
                        printf( "a error occured!\n" );
                }
                // user unknown!
                else
                {
                    // argument -u set ?
                    if( !cUUCP )
                    {
                        // unknown user to Pop3
                        printf( "Moving mail for nobody (%s) into %s's Pop3-Box...", pcUser, pcDefUsr );
                        if( !fmoveau( pcMail, pcDefBox, pcUser ) )
                        {
                            if( !unlink( pcFound ) )
                                printf( "success\n" );
                        }else
                            printf( "a error occured!\n" );
                    }
                    else
                    {
                        // unknown user to Gateway
                        printf( "Moving mail for nobody (%s) to GateWay...", pcUser );
                        
                        // get sender-name and -host from mailfile
                        pFMail = fopen( pcMail, "rb" );
                        if( pFMail )
                        {
                            // get first line
                            fgets( pcZeile , 255, pFFound );
                            // close Mailfile
                            fclose( pFMail );
            
                            // get second word in this as from-name/host.
                            pcTmp=strtok( pcZeile, " \t" );
                            pcTmp=strtok(    NULL, " \t\n\r" );
                            
                            // assuming fromUser is only local User
                            strcpy(pcFromUser,pcTmp);
                            strcpy(pcFromHost,pcHOST);
                            
                            // address is (not local) in arpa-format
                            if( strchr( pcTmp, '!' ) )
                            {
                                strcpy( pcFromUser, strchr( pcTmp, '!' )+1 );
                                // terminate string at last '!'
                                *strrchr( pcTmp, '!' )=0;
                                // are more '!' in string ?
                                if( strchr( pcTmp, '!' ) )
                                    strcpy( pcFromHost, strchr( pcTmp, '!' )+1 );
                                else
                                    strcpy( pcFromHost, pcTmp );
                            };
                            
                            // address is (not local) in arpa-format
                            if( strchr( pcTmp, '@' ) )
                            {
                                strcpy( pcFromHost, strchr( pcTmp, '@' )+1 );
                                *strchr( pcTmp, '@' )=0;
                                strcpy( pcFromUser, pcTmp );
                            };
                        };
                        
                        // initial compute D. & X.filenames
                        job = job_id(1);
                        sprintf( pcRdName, "D.%.7s0%sr", pcHOST , job );
                        sprintf( pcRxName, "X.%.7s0%sr", pcHOST , job );
                        sprintf( pcRdFile, "%s/%s", pcUSPL, pcRdName  );
                        sprintf( pcRxFile, "%s/%s", pcUSPL, pcRxName  );
                        
                        // set not unique one
                        uniq=1;
                        // unique ?
                        while( uniq )
                        {
                            // is unique
                            uniq=0;
                            // able to open file
                            pF=fopen( pcRdFile, "rb" );
                            if( pF )
                            {
                                fclose( pF );
                                // not uniq
                                uniq=1;
                            };
                            // able to open file
                            pF=fopen( pcRxFile, "rb" );
                            if( pF )
                            {
                                fclose( pF );
                                // not uniq
                                uniq=1;
                            };
                            // if not uniq compute new
                            if( uniq )
                            {
                                // compute D. & X.filenames
                                job = job_id(1);
                                sprintf( pcRdName, "D.%.7s0%sr", pcHOST , job );
                                sprintf( pcRxName, "X.%.7s0%sr", pcHOST , job );
                                sprintf( pcRdFile, "%s/%s", pcUSPL, pcRdName  );
                                sprintf( pcRxFile, "%s/%s", pcUSPL, pcRxName  );
                            };

                        };
                        
                        // Rename to D.file
                        if( !fcopyau( pcMail, pcRdFile, pcUser ) )
                        {
                            if( !unlink( pcFound ) )
                                if( !unlink( pcMail ) )
                                    printf( "success\n" );
                        }else
                            printf( "a error occured!\n" );

                        // Create X.file for rmail
                        pFRxFile = fopen( pcRxFile, "wb" );
                        if( pFRxFile )
                        {
                            fprintf( pFRxFile, "U %s %s\n", pcFromUser, pcFromHost );
                            fprintf( pFRxFile, "F %s\n", pcRdName );
                            fprintf( pFRxFile, "I %s\n", pcRdName );
                            fprintf( pFRxFile, "C rmail %s!%s\n", pcHOST, pcUser );
                            fclose( pFRxFile );
                        };
                    };
                };
            };
        }
        else
        {
            // errormsg if unsuccessfull
            printf( "   Unable to open %s ! ... exit\n\n", pcFound );
            // exit with errorlevel 3
            return(3);
        };

        // get next hit for searchmask
        rc = _dos_findnext( &fileinfo );
    };
    // no more hit
    _dos_findclose( &fileinfo );
    
    return(0);
};

char fcopyau( char *pcOrig, char *pcDest, char *pcUser )
{
    unsigned long lFSize;
    FILE          *pFOrig, *pFDest;
    
    strcln( pcOrig );
    strcln( pcDest );
    pFOrig = fopen( pcOrig, "rb" );
    if( !pFOrig )
    {
        return(1);
    }else{
        fseek( pFOrig, 0, SEEK_END );
        lFSize = ftell( pFOrig );
        fseek( pFOrig, 0, SEEK_SET );

        pFDest = fopen( pcDest, "wb" );
        if( !pFDest )
        {
            fclose( pFOrig );
            return(1);
        }else{
            fprintf( pFDest, "To: %s\n", pcUser );
            for( ; lFSize>0; lFSize-- )
                fputc( fgetc( pFOrig ), pFDest );
            fclose( pFDest );
        };
        fclose( pFOrig );
        return(0);
    }
};

char fmoveau(char *pcFile, char *pcPath, char *pcUser)
{
    if( !fcopyau_( pcFile, pcPath, pcUser) )
        if( !unlink( pcFile ) )
            return(0);
    return(1);
};

char fcopyau_(char *pcFile, char *pcPath, char *pcUser)
{
    char          pcDest[256];
    unsigned long lFSize;
    FILE          *pFFile, *pFDest;
    
    strcln( pcFile );
    pFFile = fopen( pcFile, "rb" );
    if( !pFFile )
        return(1);
    else{
        strcpy( pcDest, pcPath );
        strcat( pcDest, strrchr( pcFile, '/') );
        pFDest = fopen( pcDest, "wb" );
        if( !pFDest )
        {
            fclose( pFFile);
            return(1);
        }else{
            fseek( pFFile, 0, SEEK_END );
            lFSize = ftell( pFFile );
            fseek( pFFile, 0, SEEK_SET );
            fprintf( pFDest, "To: %s\n", pcUser );
            for( ; lFSize>0; lFSize-- )
                fputc( fgetc( pFFile ), pFDest );
            fclose( pFDest );
            fclose( pFFile );
            return(0);
        };
    };
};

char fmove(char *pcFile, char *pcPath)
{
    if( !fcopy_( pcFile, pcPath) )
        if( !unlink( pcFile ) )
            return(0);
    return(1);
};

char fcopy_(char *pcFile, char *pcPath)
{
    char          pcDest[256];
    unsigned long lFSize;
    FILE          *pFFile, *pFDest;
    
    strcln( pcFile );
    pFFile = fopen( pcFile, "rb" );
    if( !pFFile )
        return(1);
    else{
        strcpy( pcDest, pcPath );
        strcat( pcDest, strrchr( pcFile, '/') );
        pFDest = fopen( pcDest, "wb" );
        if( !pFDest )
        {
            fclose( pFFile);
            return(1);
        }else{
            fseek( pFFile, 0, SEEK_END );
            lFSize = ftell( pFFile );
            fseek( pFFile, 0, SEEK_SET );
            for( ; lFSize>0; lFSize-- )
                fputc( fgetc( pFFile ), pFDest );
            fclose( pFDest );
            fclose( pFFile );
            return(0);
        };
    };
};

void strcln(char *pc)
{
    char c;
    
    for(c=0;c<=strlen(pc);c++) /* CR raus */
	if(pc[c]==0x0D)
	    pc[c]=0x00;
    for(c=0;c<=strlen(pc);c++) /* LF raus */
	if(pc[c]==0x0A)
	    pc[c]=0x00;
    for(c=0;c<=strlen(pc);c++) /* '\' zu '/' */
	if(pc[c]==0x5C)
	    pc[c]=0x2F;
};

char *job_id( int update )
{
    static char job[4];
    static char set[] = "0123456789abcdefghijklmnopqrstuvwxyz";
    static long base = sizeof(set) - 1;
    long seq;
    size_t c;
    FILE *fp;
    
    /* Read last sequence number */
    fp = fopen("SEQF", "rt");
    if(fp) 
    {
        if(fscanf(fp, "%ld", &seq) != 1)
            seq = getpid();
        fclose(fp);
    }
    else
    {
        seq = getpid();
    };

    /* Update next sequence number */
    if(update) 
    {
        fp = fopen("SEQF", "wt");
        if(fp) 
	{
            fprintf(fp, "%ld\n", seq + 1);
            fclose(fp);
        };
    };

    /* Convert sequence to 3-char job id */
    seq %= base * base * base;
    job[c = sizeof(job) - 1] = '\0';
    while(c-- > 0) 
    {
        job[c] = set[seq % base];
        seq /= base;
    };
    return(job);
};

void intropg( void )
{
    printf( "\n" );
    printf( "  Ŀ\n"   );
    printf( "             S M T P M O V E          \n"   );
    printf( "  Ĵ\n"   );
    printf( "         A EMail-Bridge for OS/2      \n"   );
    printf( "               Version 1.0            \n"   );
    printf( "                                      \n"   );
    printf( "  Copyright (C) 1995 Manfred Ackermann\n"   );
    printf( "  \n\n" );
};

void helppg( void )
{
    printf( "   SMTPMOVE:\n\n" );
    printf( "      smtpmove.exe [-m mailaddr] [-i smtpin] [-u] [-h hostname]\n" );
    printf( "                   [-s uucpspool] [-dbg]\n\n" );
    printf( "   Purpose:\n\n" );
    printf( "      smtpmove moves the via sendmail (RFC821) received mail into the Pop3\n"  );
    printf( "      mailboxes for pick-up and/or moves them to the gateway.\n\n" );
    printf( "   Arguments:\n\n" );
    printf( "      -m    specifies the directory where the mailaddr-file exists. If\n" );
    printf( "            you don't specify it then the environment-variable MAILADDR\n" );
    printf( "            will be used\n\n" );
    printf( "      -i    specifies the directory where sendmail stores the incoming mail.\n" );
    printf( "            If you don't specify it then the environment-variable SMTPMAIL\n" );
    printf( "            will be used.\n\n" );
    printf( "      -u    switches moving mail for unknown user from the default Pop3-Box\n" );
    printf( "            to moving them to the Gateway via UUCP.\n\n" );
    printf( "      -h    specifies hostname of this maschine. If you don't specify then\n" );
    printf( "            the environment-variable HOSTNAME will be used.\n\n" );
    printf( "      -s    specifies the directory where the gateway-software scans for new\n" );
    printf( "            mail. Here are newly UUCP-files created.\n\n" );
    printf( "      -dbg  debugmode on.\n\n" );
    printf( "   e.g.: smtpmove -i c:\\tcpip\\etc\\mail -r c:\\tcpip\\pop3d\n" );
};